#!/bin/env python3
"""kmljoin - Join multiple KML files into a single KML file."""


import copy
import sys
import xml.etree.ElementTree as ET


XML_NS = {
    '': 'http://www.opengis.net/kml/2.2',
    'gx': 'http://www.google.com/kml/ext/2.2',
}

for nskey, ns in XML_NS.items():
    ET.register_namespace(nskey, ns)

def combine_files(kml_files):
    """Combine two or more KML files into one."""
    if not kml_files or len(kml_files) < 2:
        raise(Exception('Two or more KML files required.'))
    (first_kml_etree, first_track, first_arrays) = _read_file(kml_files[0])
    for kml_file in kml_files[1:]:
        (kml_etree, track, arrays) = _read_file(kml_file)
        # concatenate Tracks
        for track_node in track:
            if track_node.tag in (f'{{{XML_NS[""]}}}when', f'{{{XML_NS["gx"]}}}coord'):
                first_track.append(copy.deepcopy(track_node))
        # concatenate SimpleArrayData arrays
        for name, array in arrays.items():
            if name in first_arrays:
                for array_node in array:
                    first_arrays[name].append(copy.deepcopy(array_node))
            else:
                first_arrays[name] = array
    return first_kml_etree


def _read_file(kml_file):
    """Parse a KML file and return the ETree, Track, and SimpleArrayData lists."""
    kml_etree = ET.parse(kml_file)

    def _get_arrays(kml_etree):
        """Extract all SimpleArrayData lists."""
        arrays = kml_etree.findall(".//gx:SimpleArrayData", XML_NS)
        return {a.get('name'): a for a in arrays}

    def _get_track(kml_etree):
        """Extract one gx:Track."""
        tracks = kml_etree.findall(f".//gx:Track", XML_NS)
        if not tracks:
            raise(Exception('No tracks found.'))
        if len(tracks) > 1:
            raise(Exception('Cannot handle multiple tracks.'))
        return tracks[0]

    return kml_etree, _get_track(kml_etree), _get_arrays(kml_etree)

if __name__ == '__main__':
    kml_files = sys.argv[1:]
    try:
        combined_etree = combine_files(kml_files)
        ET.dump(combined_etree)
    except Exception as e:
        print('Error: {}'.format(e))

